@@ -247,15 +247,8 @@ struct pam_message {
        unsigned char data[length];
    };
 
-   We define the following union to access it:
- */
-
-union pam_u_packet_p {
-    int *length;
-    unsigned char *data;
-    char *text;
-    const char *prompt;
-};
+   The 'libpam_client' library is designed around this flavor of
+   message and should be used to handle this flavor of msg_style. */
 
 /* Used to return the user's response to the PAM library.  This
    structure is allocated by the application program, and free()'d by
