@@ -33,7 +33,7 @@
 #define _PAM_PRIVATE_H
 
 /* this is not used at the moment --- AGM */
-#define LIBPAM_VERSION 56
+#define LIBPAM_VERSION 59
 
 #include <security/pam_appl.h>
 #include <security/pam_modules.h>
@@ -60,7 +60,7 @@
 struct handler {
     int must_fail;
     int (*func)(pam_handle_t *pamh, int flags, int argc, char **argv);
-    int control_flag;
+    int actions[_PAM_RETURN_VALUES];
     int argc;
     char **argv;
     struct handler *next;
@@ -144,11 +144,20 @@ struct pam_handle {
 #define PAM_CLOSE_SESSION 4
 #define PAM_CHAUTHTOK     5
 
-/* Values for control_flag in struct handler */
-#define PAM_F_REQUIRED    0
-#define PAM_F_SUFFICIENT  1
-#define PAM_F_OPTIONAL    2
-#define PAM_F_CRITICAL    3
+#define _PAM_ACTION_IS_JUMP(x)  ((x) > 0)
+#define _PAM_ACTION_IGNORE      0
+#define _PAM_ACTION_OK         -1
+#define _PAM_ACTION_DONE       -2
+#define _PAM_ACTION_BAD        -3
+#define _PAM_ACTION_DIE        -4
+#define _PAM_ACTION_RESET      -5
+/* Add any new entries here.  Will need to change ..._UNDEF and then
+ * need to change pam_tokens.h */
+#define _PAM_ACTION_UNDEF      -6      /* this is treated as an error (==-3) */
+
+/* character tables for parsing config files */
+extern const char * const _pam_token_actions[-_PAM_ACTION_UNDEF];
+extern const char * const _pam_token_returns[_PAM_RETURN_VALUES+1];
 
 /*
  * internally defined functions --- these should not be directly
@@ -230,6 +239,10 @@ void _pam_log_error(const char *format, ...);
 
 void _pam_sanitize(pam_handle_t *pamh);
 
+void _pam_set_default_control(int *control_array, int default_action);
+
+void _pam_parse_control(int *control_array, char *tok);
+
 /*
  * XXX - Take care with this. It could confuse the logic of a trailing
  *       else
