@@ -93,6 +93,10 @@ int innetgr(const char *, const char *, const char *,const char *);
 #include <security/pam_modules.h>
 #include <security/_pam_macros.h>
 
+/* to the best of my knowledge, all modern UNIX boxes have 32 bit integers */
+#define U32 unsigned int
+
+
 /*
  * Options for this module
  */
@@ -230,11 +234,11 @@ static int pam_get_ruser(pam_handle_t *pamh, const char **ruser
  */
 
 static int
-__icheckhost (pam_handle_t *pamh, struct _options *opts, u_long raddr
+__icheckhost (pam_handle_t *pamh, struct _options *opts, U32 raddr
 	      , register char *lhost, const char *rhost)
 {
     struct hostent *hp;
-    u_long laddr;
+    U32 laddr;
     int negate=1;    /* Multiply return with this to get -1 instead of 1 */
     char **pp, *user;
 
@@ -270,7 +274,7 @@ __icheckhost (pam_handle_t *pamh, struct _options *opts, u_long raddr
     
     /* Spin through ip addresses. */
     for (pp = hp->h_addr_list; *pp; ++pp)
-	if (!memcmp (&raddr, *pp, sizeof (unsigned long)))
+	if (!memcmp (&raddr, *pp, sizeof (U32)))
 	    return (negate);
 
     /* No match. */
@@ -334,17 +338,17 @@ static int __isempty(char *p)
 
 static int
 __ivaliduser (pam_handle_t *pamh, struct _options *opts,
-	      FILE *hostf, u_long raddr,
+	      FILE *hostf, U32 raddr,
 	      const char *luser, const char *ruser, const char *rhost)
 {
     register const char *user;
     register char *p;
-    int len, hcheck, ucheck;
+    int hcheck, ucheck;
     char buf[MAXHOSTNAMELEN + 128];                       /* host + login */
 
     buf[sizeof (buf)-1] = '\0';                 	/* terminate line */
 
-    while ((len = fgets(buf, sizeof(buf), hostf))) {   /* hostf file line */
+    while (fgets(buf, sizeof(buf), hostf) != NULL) {   /* hostf file line */
         p = buf;                              /* from beginning of file.. */
 
 	/* Skip empty or comment lines */
@@ -353,7 +357,7 @@ __ivaliduser (pam_handle_t *pamh, struct _options *opts,
 	}
 
 	/* Skip lines that are too long. */
-	if (len == sizeof(buf) && strchr(p, '\n') == NULL) {
+	if (strchr(p, '\n') == NULL) {
 	    int ch = getc(hostf);
 
 	    while (ch != '\n' && ch != EOF)
@@ -429,7 +433,7 @@ __ivaliduser (pam_handle_t *pamh, struct _options *opts,
 
 static int
 pam_iruserok(pam_handle_t *pamh,
-	 struct _options *opts, u_long raddr, int superuser,
+	 struct _options *opts, U32 raddr, int superuser,
 	 const char *ruser, const char *luser, const char *rhost)
 {
     const char *cp;
@@ -584,13 +588,20 @@ pam_ruserok (pam_handle_t *pamh,
 {
     struct hostent *hp;
     int answer = 1;                             /* default to failure */
-    u_long *addrs;
+    U32 *addrs;
     int n, i;
 
     opts->last_error = (char *) 0;
     hp               = gethostbyname(rhost);         /* identify host */
 
     if (hp != NULL) {
+	/* First of all check the address length */
+        if (hp->h_length != 4) {
+            _pam_log(LOG_ALERT, "pam_rhosts module can't work with not IPv4 "
+		     "addresses");
+            return 1;                                    /* not allowed */
+        }
+
         /* loop though address list */
         for (n = 0; hp->h_addr_list[n]; n++);
         D(("rhosts: %d addresses", n))
@@ -712,6 +723,11 @@ int pam_sm_authenticate (pam_handle_t *pamh,
 {
     int retval;
 
+    if (sizeof(U32) != 4) {
+        _pam_log (LOG_ALERT, "pam_rhosts module can\'t work on this hardware "
+		  "(yet)");
+        return PAM_AUTH_ERR;
+    }
     sethostent(1);
     retval = _pam_auth_rhosts (pamh, flags, argc, argv);
     endhostent();
