@@ -337,7 +337,7 @@ typedef struct {
 struct day {
      const char *d;
      int bit;
-} const days[11] = {
+} static const days[11] = {
      { "su", 01 },
      { "mo", 02 },
      { "tu", 04 },
